﻿using System;
using System.Collections.ObjectModel;
using System.Net;
using System.Web.UI;
using PdfSharp;
using PdfSharp.Pdf;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;

namespace WebPost9941
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Map1.MapUnit = GeographyUnit.DecimalDegree;

                WmsOverlay wmsOverlay = new WmsOverlay("WmsOverlay", new Uri("http://vmap0.tiles.osgeo.org/wms/vmap0"));
                wmsOverlay.Parameters.Add("layers", "basic");
                wmsOverlay.Parameters.Add("styles", "default");
                Map1.CustomOverlays.Add(wmsOverlay);

                Map1.CurrentExtent = new RectangleShape(-130, 90, 130, -90);
            }
        }

        protected void ToPdfClick(object sender, EventArgs e)
        {
            WmsOverlay wmsOverlay = (WmsOverlay)Map1.CustomOverlays["WmsOverlay"];

            var dcmt = new PdfDocument();
            var page = dcmt.AddPage();
            page.Orientation = PageOrientation.Portrait;

            var pdfGeoCanvas = new PdfGeoCanvas();
            pdfGeoCanvas.BeginDrawing(page, Map1.CurrentExtent, GeographyUnit.DecimalDegree);
            var wmsLayer = wmsOverlay.ToLayer();

            // this event is unnecessary for normal cases. 
            // the wms layer will parse the wms server's capability first and get the server uri; 
            // then merge the request uri by the Uri in the capacility.

            // this sample server's capability doesn't have vmap0 section so it requests failed. 
            // here I just add the section to make it a valid parameter.

            // the overlay doesn't need it because it merges the uri by the server address by user settings directly.
            wmsLayer.SendingWebRequest += (s, args) =>
            {
                args.WebRequest = WebRequest.Create(new Uri(args.WebRequest.RequestUri.AbsoluteUri
                    .Replace("?", "/vmap0?")));
            };

            wmsLayer.Open();
            wmsLayer.Draw(pdfGeoCanvas, new Collection<SimpleCandidate>());
            wmsLayer.Close();
            pdfGeoCanvas.EndDrawing();

            string filename = Server.MapPath(".") + "\\MapSuite PDF Map.pdf";
            dcmt.Save(filename);
        }
    }

    public static class WmsOverlayExtentsion
    {
        public static WmsRasterLayer ToLayer(this WmsOverlay wmsOverlay)
        {
            WmsRasterLayer wmsLayer = null;
            if (wmsOverlay.ServerUris.Count == 0)
            {
                wmsLayer = new WmsRasterLayer();
            }
            else
            {
                wmsLayer = new WmsRasterLayer(wmsOverlay.ServerUris[0]);
                foreach (var parameter in wmsOverlay.Parameters)
                {
                    switch (parameter.Key.ToUpper())
                    {
                        case "LAYERS":
                            wmsLayer.ActiveLayerNames.Add(parameter.Value);
                            break;
                        case "STYLES":
                            wmsLayer.ActiveStyleNames.Add(parameter.Value);
                            break;
                        // if you have other parameters, please parse it here.
                        default:
                            wmsLayer.Parameters.Add(parameter.Key, parameter.Value);
                            break;
                    }
                }
            }

            return wmsLayer;
        }
    }
}
